package revealinos;

import java.io.IOException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.actions.ActionDelegate;

public class RevealInOS extends ActionDelegate {

    private ISelection m_selection;

    private enum OS { Windows, Mac, Other }
    private final static OS m_os = computeOS();

    public void init(IAction action) {
    	String actionText = null;

    	switch (m_os) {
    	case Windows:
    		actionText = Messages.getString("RevealInOS.showInWindowsExplorer"); //$NON-NLS-1$
    		break;
    	case Mac:
    		actionText = Messages.getString("RevealInOS.revealInFinder"); //$NON-NLS-1$
    		break;
    	}

    	if (actionText != null) {
    		action.setText(actionText);
    	}
    }

    private static OS computeOS() {
		String os = Platform.getOS();
		if (Platform.OS_WIN32.equals(os)) {
			return OS.Windows;
		} else if (Platform.OS_MACOSX.equals(os)) {
			return OS.Mac;
		} else {
			return OS.Other;
		}
    }

	public void run(IAction action) {
		if (m_selection instanceof IStructuredSelection) {
			IStructuredSelection ss = (IStructuredSelection) m_selection;
			if (ss.size() == 1) {
				Object firstElement = ss.getFirstElement();
				if (firstElement instanceof IResource) {
					IResource resource = (IResource) firstElement;
					IPath location = resource.getLocation();
					if (location != null) {
						String os = Platform.getOS();
						try {
							if (Platform.OS_WIN32.equals(os)) {
								Runtime.getRuntime().exec(
									new String[] {
										"explorer.exe", //$NON-NLS-1$
										"/select," + location.toOSString() //$NON-NLS-1$
									} );
							} else if (Platform.OS_MACOSX.equals(os)) {
								Runtime.getRuntime().exec(
									new String[] {
										"/usr/bin/osascript", //$NON-NLS-1$
										"-e", //$NON-NLS-1$
										"tell application \"Finder\"", //$NON-NLS-1$
										"-e", //$NON-NLS-1$
										"reveal POSIX file \"" + location.toOSString() + "\"", //$NON-NLS-1$ //$NON-NLS-2$
										"-e", //$NON-NLS-1$
										"activate", //$NON-NLS-1$
										"-e", //$NON-NLS-1$
										"end tell" //$NON-NLS-1$
									} );
							}
						} catch (IOException e) {
							MessageDialog.openError(Display.getCurrent().getActiveShell(),
									"Error", e.getLocalizedMessage()); //$NON-NLS-1$
						}
					}
				}
			}
		}
    }

    public void selectionChanged(IAction action, ISelection selection) {
		m_selection = selection;
    }

    public void dispose() {
    	m_selection = null;
    }
}
